@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #define VIDEO_WINMAIN_ARGS
@@ -32,17 +24,11 @@
 #include "tbb/tick_count.h"
 #include "../../common/utility/utility.h"
 
-#if __TBB_MIC
-#pragma offload_attribute (push,target(mic))
-#endif // __TBB_MIC
 #include "seismic_video.h"
 #include "universe.h"
 #include "tbb/task_scheduler_init.h"
 
 Universe u;
-#if __TBB_MIC
-#pragma offload_attribute (pop)
-#endif // __TBB_MIC
 
 struct RunOptions {
     //! It is used for console mode for test with different number of threads and also has
@@ -60,9 +46,9 @@ struct RunOptions {
 
 int do_get_default_num_threads() {
     int threads;
-    #if __TBB_MIC
+#if __TBB_MIC_OFFLOAD
     #pragma offload target(mic) out(threads)
-    #endif // __TBB_MIC
+#endif // __TBB_MIC_OFFLOAD
     threads = tbb::task_scheduler_init::default_num_threads();
     return threads;
 }
@@ -76,15 +62,15 @@ RunOptions ParseCommandLine(int argc, char *argv[]){
     // zero number of threads means to run serial version
     utility::thread_number_range threads(get_default_num_threads,0,get_default_num_threads());
 
-    int numberOfFrames = 1000;
+    int numberOfFrames = 0;
     bool silent = false;
     bool serial = false;
 
     utility::parse_cli_arguments(argc,argv,
         utility::cli_argument_pack()
-            //"-h" option for for displaying help is present implicitly
+            //"-h" option for displaying help is present implicitly
             .positional_arg(threads,"n-of-threads",utility::thread_number_range_desc)
-            .positional_arg(numberOfFrames,"n-of-frames","number of frames the example processes internally")
+            .positional_arg(numberOfFrames,"n-of-frames","number of frames the example processes internally (0 means unlimited)")
             .arg(silent,"silent","no output except elapsed time")
             .arg(serial,"serial","in GUI mode start with serial version of algorithm")
     );
@@ -109,11 +95,15 @@ int main(int argc, char *argv[])
         }
         else if(video.init_console()) {
             // do console mode
+            if (options.numberOfFrames == 0) {
+                options.numberOfFrames = 1000;
+                std::cout << "Substituting 1000 for unlimited frames because not running interactively\n";
+            }
             for(int p = options.threads.first;  p <= options.threads.last; p = options.threads.step(p)) {
                 tbb::tick_count xwayParallelismStartTime = tbb::tick_count::now();
                 u.InitializeUniverse(video);
                 int numberOfFrames = options.numberOfFrames;
-                #if __TBB_MIC
+#if __TBB_MIC_OFFLOAD
                 drawing_memory dmem = video.get_drawing_memory();
                 char *pMem = dmem.get_address();
                 size_t memSize = dmem.get_size();
@@ -124,21 +114,21 @@ int main(int argc, char *argv[])
                     // since the address spaces on host and on target are different
                     dmem.set_address(pMem);
                     u.SetDrawingMemory(dmem);
-                #endif // __TBB_MIC
-                if (p==0){
-                    //run a serial version
-                    for( int i=0; i<numberOfFrames; ++i ){
-                        u.SerialUpdateUniverse();
-                    }
-                }else{
-                    tbb::task_scheduler_init init(p);
-                    for( int i=0; i<numberOfFrames; ++i ){
-                        u.ParallelUpdateUniverse();
+#endif // __TBB_MIC_OFFLOAD
+                    if (p==0) {
+                        //run a serial version
+                        for( int i=0; i<numberOfFrames; ++i ) {
+                            u.SerialUpdateUniverse();
+                        }
+                    } else {
+                        tbb::task_scheduler_init init(p);
+                        for( int i=0; i<numberOfFrames; ++i ) {
+                            u.ParallelUpdateUniverse();
+                        }
                     }
+#if __TBB_MIC_OFFLOAD
                 }
-                #if __TBB_MIC
-                }
-                #endif // __TBB_MIC
+#endif // __TBB_MIC_OFFLOAD
 
                 if (!options.silent){
                     double fps =  options.numberOfFrames/((tbb::tick_count::now()-xwayParallelismStartTime).seconds());
