@@ -1,29 +1,21 @@
 /*
     Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 const unsigned MByte = 1024*1024;
@@ -34,7 +26,7 @@ bool __tbb_test_errno = false;
 #if __TBB_WIN8UI_SUPPORT	
 // testing allocator itself not iterfaces
 // so we can use desktop functions
-#define _CRT_USE_WINAPI_FAMILY_DESKTOP_APP 1
+#define _CRT_USE_WINAPI_FAMILY_DESKTOP_APP !_M_ARM
 #define HARNESS_NO_PARSE_COMMAND_LINE 1
 #include "harness.h"
 // FIXME: fix the test to support New Windows *8 Store Apps mode.
@@ -93,9 +85,15 @@ void limitMem( size_t limit )
 void limitMem( size_t limit )
 {
     rlimit rlim;
-    rlim.rlim_cur = limit? limit*MByte : (rlim_t)RLIM_INFINITY;
-    rlim.rlim_max = (rlim_t)RLIM_INFINITY;
-    int ret = setrlimit(RLIMIT_AS,&rlim);
+    int ret = getrlimit(RLIMIT_AS,&rlim);
+    if (0 != ret) {
+        REPORT("getrlimit() returned an error: errno %d\n", errno);
+        exit(1);
+    }
+    if (rlim.rlim_max==(rlim_t)RLIM_INFINITY)
+        rlim.rlim_cur = (limit > 0) ? limit*MByte : rlim.rlim_max;
+    else rlim.rlim_cur = (limit > 0 && limit<rlim.rlim_max) ? limit*MByte : rlim.rlim_max;
+    ret = setrlimit(RLIMIT_AS,&rlim);
     if (0 != ret) {
         REPORT("Can't set limits: errno %d\n", errno);
         exit(1);
@@ -142,8 +140,6 @@ extern "C" void *__cdecl _aligned_malloc(size_t,size_t);
     #pragma warning (pop)
 #endif
 
-const size_t COUNT_ELEM_CALLOC = 2;
-const int COUNT_TESTS = 1000;
 const int COUNT_ELEM = 25000;
 const size_t MAX_SIZE = 1000;
 const int COUNTEXPERIMENT = 10000;
@@ -184,8 +180,10 @@ TestAlignedRealloc* Raligned_realloc;
 bool error_occurred = false;
 
 #if __APPLE__
-// Tests that use the variable are skipped on OS X*
+// Tests that use the variables are skipped on OS X*
 #else
+const size_t COUNT_ELEM_CALLOC = 2;
+const int COUNT_TESTS = 1000;
 static bool perProcessLimits = true;
 #endif
 
@@ -1034,14 +1032,12 @@ void CMemTest::RunAllTests(int total_threads)
     if (Raligned_realloc)
         InvariantDataRealloc(/*aligned=*/true);
     TestAlignedParameters();
+    UniquePointer();
+    AddrArifm();
 #if __APPLE__
     REPORT("Known issue: some tests are skipped on OS X*\n");
 #else
-    UniquePointer();
-    AddrArifm();
-#if !__TBB_MIC_NATIVE
     NULLReturn(1*MByte,100*MByte,total_threads);
-#endif
 #endif
     if (FullLog) REPORT("Tests for %d threads ended\n", total_threads);
 }
